@@ -51,7 +51,6 @@ HINSTANCE hInst;
 DWORD mshtml_tls = TLS_OUT_OF_INDEXES;
 
 static HINSTANCE shdoclc = NULL;
-static HDC display_dc;
 static WCHAR *status_strings[IDS_STATUS_LAST-IDS_STATUS_FIRST+1];
 static IMultiLanguage2 *mlang;
 
@@ -140,8 +139,6 @@ static void process_detach(void)
         FreeLibrary(shdoclc);
     if(mshtml_tls != TLS_OUT_OF_INDEXES)
         TlsFree(mshtml_tls);
-    if(display_dc)
-        DeleteObject(display_dc);
     if(mlang)
         IMultiLanguage2_Release(mlang);
 
@@ -210,21 +207,6 @@ HINSTANCE get_shdoclc(void)
     return shdoclc = LoadLibraryExW(wszShdoclc, NULL, LOAD_LIBRARY_AS_DATAFILE);
 }
 
-HDC get_display_dc(void)
-{
-    static const WCHAR displayW[] = {'D','I','S','P','L','A','Y',0};
-
-    if(!display_dc) {
-        HDC hdc;
-
-        hdc = CreateICW(displayW, NULL, NULL, NULL);
-        if(InterlockedCompareExchangePointer((void**)&display_dc, hdc, NULL))
-            DeleteObject(hdc);
-    }
-
-    return display_dc;
-}
-
 BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID reserved)
 {
     switch(fdwReason) {
@@ -589,6 +571,8 @@ const char *debugstr_mshtml_guid(const GUID *iid)
     X(IID_IHTMLPrivateWindow);
     X(IID_IHtmlLoadOptions);
     X(IID_IInternetHostSecurityManager);
+    X(IID_IInternetProtocol);
+    X(IID_IInternetProtocolRoot);
     X(IID_IManagedObject);
     X(IID_IMarshal);
     X(IID_IMonikerProp);
