@@ -293,6 +293,24 @@ __ASM_GLOBAL_FUNC( set_cpu_context,
                    "pop {pc}" )
 
 
+/***********************************************************************
+ *           get_server_context_flags
+ *
+ * Convert CPU-specific flags to generic server flags
+ */
+static unsigned int get_server_context_flags( DWORD flags )
+{
+    unsigned int ret = 0;
+
+    flags &= ~CONTEXT_ARM;  /* get rid of CPU id */
+    if (flags & CONTEXT_CONTROL) ret |= SERVER_CTX_CONTROL;
+    if (flags & CONTEXT_INTEGER) ret |= SERVER_CTX_INTEGER;
+    if (flags & CONTEXT_FLOATING_POINT) ret |= SERVER_CTX_FLOATING_POINT;
+    if (flags & CONTEXT_DEBUG_REGISTERS) ret |= SERVER_CTX_DEBUG_REGISTERS;
+    return ret;
+}
+
+
 /***********************************************************************
  *           copy_context
  *
@@ -449,8 +467,10 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
 {
     NTSTATUS ret;
     BOOL self;
+    context_t server_context;
 
-    ret = set_thread_context( handle, context, &self );
+    context_to_server( &server_context, context );
+    ret = set_thread_context( handle, &server_context, &self );
     if (self && ret == STATUS_SUCCESS) set_cpu_context( context );
     return ret;
 }
@@ -468,7 +488,11 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 
     if (!self)
     {
-        if ((ret = get_thread_context( handle, context, &self ))) return ret;
+        context_t server_context;
+        unsigned int server_flags = get_server_context_flags( context->ContextFlags );
+
+        if ((ret = get_thread_context( handle, &server_context, server_flags, &self ))) return ret;
+        if ((ret = context_from_server( context, &server_context ))) return ret;
         needed_flags &= ~context->ContextFlags;
     }
 
@@ -1025,15 +1049,6 @@ void signal_init_process(void)
 }
 
 
-/**********************************************************************
- *              __wine_enter_vm86   (NTDLL.@)
- */
-void __wine_enter_vm86( CONTEXT *context )
-{
-    MESSAGE("vm86 mode not supported on this platform\n");
-}
-
-
 /**********************************************************************
  *              RtlAddFunctionTable   (NTDLL.@)
  */
@@ -1228,24 +1243,17 @@ __ASM_GLOBAL_FUNC( start_thread,
                    /* store exit frame */
                    "ldr r4, [sp, #40]\n\t"    /* teb */
                    "str sp, [r4, #0x1d4]\n\t" /* teb->SystemReserved2 */
-                   /* build initial context on thread stack */
-                   "ldr r4, [r4, #4]\n\t"     /* teb->Tib.StackBase */
-                   "sub r5, r4, #0x1a0\n\t"   /* sizeof(CONTEXT) */
-                   "mov ip, #0x0200000\n\t"   /* CONTEXT_ARM */
-                   "add ip, #0x3\n\t"         /* CONTEXT_FULL */
-                   "str ip, [r5]\n\t"         /* context->ContextFlags */
-                   "str r0, [r5, #0x4]\n\t"   /* context->R0 = entry */
-                   "str r1, [r5, #0x8]\n\t"   /* context->R1 = arg */
-                   "str r4, [r5, #0x38]\n\t"  /* context->Sp = stack */
-                   "str r3, [r5, #0x40]\n\t"  /* context->Pc = relay */
                    /* switch to thread stack */
-                   "mov sp, r5\n\t"
+                   "ldr r4, [r4, #4]\n\t"     /* teb->Tib.StackBase */
+                   "sub sp, r4, #0x1000\n\t"
                    /* attach dlls */
-                   "mov r0, r5\n\t"  /* context */
-                   "mov r1, r2\n\t"  /* suspend */
-                   "bl " __ASM_NAME("attach_dlls") "\n\t"
+                   "bl " __ASM_NAME("attach_thread") "\n\t"
+                   "mov sp, r0\n\t"
+                   /* clear the stack */
+                   "and r0, #~0xff0\n\t"  /* round down to page size */
+                   "bl " __ASM_NAME("virtual_clear_thread_stack") "\n\t"
                    /* switch to the initial context */
-                   "mov r0, r5\n\t"
+                   "mov r0, sp\n\t"
                    "b " __ASM_NAME("set_cpu_context") )
 
 extern void DECLSPEC_NORETURN call_thread_exit_func( int status, void (*func)(int), TEB *teb );
@@ -1258,6 +1266,46 @@ __ASM_GLOBAL_FUNC( call_thread_exit_func,
                    "movne sp, r3\n\t"
                    "blx r1" )
 
+/***********************************************************************
+ *           init_thread_context
+ */
+static void init_thread_context( CONTEXT *context, LPTHREAD_START_ROUTINE entry, void *arg, void *relay )
+{
+    context->R0 = (DWORD)entry;
+    context->R1 = (DWORD)arg;
+    context->Sp = (DWORD)NtCurrentTeb()->Tib.StackBase;
+    context->Pc = (DWORD)relay;
+}
+
+
+/***********************************************************************
+ *           attach_thread
+ */
+PCONTEXT DECLSPEC_HIDDEN attach_thread( LPTHREAD_START_ROUTINE entry, void *arg,
+                                        BOOL suspend, void *relay )
+{
+    CONTEXT *ctx;
+
+    if (suspend)
+    {
+        CONTEXT context = { CONTEXT_ALL };
+
+        init_thread_context( &context, entry, arg, relay );
+        wait_suspend( &context );
+        ctx = (CONTEXT *)((ULONG_PTR)context.Sp & ~15) - 1;
+        *ctx = context;
+    }
+    else
+    {
+        ctx = (CONTEXT *)NtCurrentTeb()->Tib.StackBase - 1;
+        init_thread_context( ctx, entry, arg, relay );
+    }
+    ctx->ContextFlags = CONTEXT_FULL;
+    attach_dlls( ctx, (void **)&ctx->R0 );
+    return ctx;
+}
+
+
 /***********************************************************************
  *           signal_start_thread
  *
