@@ -123,8 +123,8 @@ static void save_context( CONTEXT *context, const ucontext_t *sigcontext )
     DWORD i;
 
     context->ContextFlags = CONTEXT_FULL;
-    context->Fp     = FP_sig(sigcontext);     /* Frame pointer */
-    context->Lr     = LR_sig(sigcontext);     /* Link register */
+    context->u.s.Fp = FP_sig(sigcontext);     /* Frame pointer */
+    context->u.s.Lr = LR_sig(sigcontext);     /* Link register */
     context->Sp     = SP_sig(sigcontext);     /* Stack pointer */
     context->Pc     = PC_sig(sigcontext);     /* Program Counter */
     context->Cpsr   = PSTATE_sig(sigcontext); /* Current State Register */
@@ -141,8 +141,8 @@ static void restore_context( const CONTEXT *context, ucontext_t *sigcontext )
 {
     DWORD i;
 
-    FP_sig(sigcontext)     = context->Fp;     /* Frame pointer */
-    LR_sig(sigcontext)     = context->Lr;     /* Link register */
+    FP_sig(sigcontext)     = context->u.s.Fp; /* Frame pointer */
+    LR_sig(sigcontext)     = context->u.s.Lr; /* Link register */
     SP_sig(sigcontext)     = context->Sp;     /* Stack pointer */
     PC_sig(sigcontext)     = context->Pc;     /* Program Counter */
     PSTATE_sig(sigcontext) = context->Cpsr;   /* Current State Register */
@@ -231,6 +231,23 @@ static void set_cpu_context( const CONTEXT *context )
     FIXME( "Not implemented on ARM64\n" );
 }
 
+/***********************************************************************
+ *           get_server_context_flags
+ *
+ * Convert CPU-specific flags to generic server flags
+ */
+static unsigned int get_server_context_flags( DWORD flags )
+{
+    unsigned int ret = 0;
+
+    flags &= ~CONTEXT_ARM64;  /* get rid of CPU id */
+    if (flags & CONTEXT_CONTROL) ret |= SERVER_CTX_CONTROL;
+    if (flags & CONTEXT_INTEGER) ret |= SERVER_CTX_INTEGER;
+    if (flags & CONTEXT_FLOATING_POINT) ret |= SERVER_CTX_FLOATING_POINT;
+    if (flags & CONTEXT_DEBUG_REGISTERS) ret |= SERVER_CTX_DEBUG_REGISTERS;
+    return ret;
+}
+
 /***********************************************************************
  *           copy_context
  *
@@ -241,8 +258,8 @@ static void copy_context( CONTEXT *to, const CONTEXT *from, DWORD flags )
     flags &= ~CONTEXT_ARM64;  /* get rid of CPU id */
     if (flags & CONTEXT_CONTROL)
     {
-        to->Fp      = from->Fp;
-        to->Lr      = from->Lr;
+        to->u.s.Fp  = from->u.s.Fp;
+        to->u.s.Lr  = from->u.s.Lr;
         to->Sp      = from->Sp;
         to->Pc      = from->Pc;
         to->Cpsr    = from->Cpsr;
@@ -281,8 +298,8 @@ NTSTATUS context_to_server( context_t *to, const CONTEXT *from )
     if (flags & CONTEXT_CONTROL)
     {
         to->flags |= SERVER_CTX_CONTROL;
-        to->integer.arm64_regs.x[29] = from->Fp;
-        to->integer.arm64_regs.x[30] = from->Lr;
+        to->integer.arm64_regs.x[29] = from->u.s.Fp;
+        to->integer.arm64_regs.x[30] = from->u.s.Lr;
         to->ctl.arm64_regs.sp     = from->Sp;
         to->ctl.arm64_regs.pc     = from->Pc;
         to->ctl.arm64_regs.pstate = from->Cpsr;
@@ -326,8 +343,8 @@ NTSTATUS context_from_server( CONTEXT *to, const context_t *from )
     if (from->flags & SERVER_CTX_CONTROL)
     {
         to->ContextFlags |= CONTEXT_CONTROL;
-        to->Fp     = from->integer.arm64_regs.x[29];
-        to->Lr     = from->integer.arm64_regs.x[30];
+        to->u.s.Fp = from->integer.arm64_regs.x[29];
+        to->u.s.Lr = from->integer.arm64_regs.x[30];
         to->Sp     = from->ctl.arm64_regs.sp;
         to->Pc     = from->ctl.arm64_regs.pc;
         to->Cpsr   = from->ctl.arm64_regs.pstate;
@@ -363,8 +380,10 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
 {
     NTSTATUS ret;
     BOOL self;
+    context_t server_context;
 
-    ret = set_thread_context( handle, context, &self );
+    context_to_server( &server_context, context );
+    ret = set_thread_context( handle, &server_context, &self );
     if (self && ret == STATUS_SUCCESS) set_cpu_context( context );
     return ret;
 }
@@ -382,7 +401,11 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 
     if (!self)
     {
-        if ((ret = get_thread_context( handle, context, &self ))) return ret;
+        context_t server_context;
+        unsigned int server_flags = get_server_context_flags( context->ContextFlags );
+
+        if ((ret = get_thread_context( handle, &server_context, server_flags, &self ))) return ret;
+        if ((ret = context_from_server( context, &server_context ))) return ret;
         needed_flags &= ~context->ContextFlags;
     }
 
@@ -568,6 +591,23 @@ static NTSTATUS raise_exception( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL f
     return STATUS_SUCCESS;
 }
 
+static inline DWORD is_write_fault( ucontext_t *context )
+{
+    DWORD inst = *(DWORD *)PC_sig(context);
+    if ((inst & 0xbfff0000) == 0x0c000000   /* C3.3.1 */ ||
+        (inst & 0xbfe00000) == 0x0c800000   /* C3.3.2 */ ||
+        (inst & 0xbfdf0000) == 0x0d000000   /* C3.3.3 */ ||
+        (inst & 0xbfc00000) == 0x0d800000   /* C3.3.4 */ ||
+        (inst & 0x3f400000) == 0x08000000   /* C3.3.6 */ ||
+        (inst & 0x3bc00000) == 0x38000000   /* C3.3.8-12 */ ||
+        (inst & 0x3fe00000) == 0x3c800000   /* C3.3.8-12 128bit */ ||
+        (inst & 0x3bc00000) == 0x39000000   /* C3.3.13 */ ||
+        (inst & 0x3fc00000) == 0x3d800000   /* C3.3.13 128bit */ ||
+        (inst & 0x3a400000) == 0x28000000)  /* C3.3.7,14-16 */
+        return EXCEPTION_WRITE_FAULT;
+    return EXCEPTION_READ_FAULT;
+}
+
 /**********************************************************************
  *		segv_handler
  *
@@ -604,8 +644,7 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
     case SIGSEGV:  /* Segmentation fault */
         rec->ExceptionCode = EXCEPTION_ACCESS_VIOLATION;
         rec->NumberParameters = 2;
-        /* FIXME: Currently the kernel provides no way to determine if it's read or write */
-        rec->ExceptionInformation[0] = 0;
+        rec->ExceptionInformation[0] = is_write_fault(context);
         rec->ExceptionInformation[1] = (ULONG_PTR)info->si_addr;
         break;
     case SIGBUS:  /* Alignment check exception */
@@ -908,14 +947,6 @@ void signal_init_process(void)
 }
 
 
-/**********************************************************************
- *              __wine_enter_vm86   (NTDLL.@)
- */
-void __wine_enter_vm86( CONTEXT *context )
-{
-    MESSAGE("vm86 mode not supported on this platform\n");
-}
-
 /***********************************************************************
  *            RtlUnwind  (NTDLL.@)
  */
@@ -1000,7 +1031,8 @@ static void thread_startup( void *param )
     context.Sp = (DWORD_PTR)NtCurrentTeb()->Tib.StackBase;
     context.Pc = (DWORD_PTR)info->start;
 
-    attach_dlls( &context, info->suspend );
+    if (info->suspend) wait_suspend( &context );
+    attach_dlls( &context, (void **)&context.u.s.X0 );
 
     ((thread_start_func)context.Pc)( (LPTHREAD_START_ROUTINE)context.u.s.X0, (void *)context.u.s.X1 );
 }
