@@ -38,7 +38,7 @@ extern struct env_stack *pushd_directories;
 
 BATCH_CONTEXT *context = NULL;
 DWORD errorlevel;
-WCHAR quals[MAXSTRING], param1[MAXSTRING], param2[MAXSTRING];
+WCHAR quals[MAX_PATH], param1[MAXSTRING], param2[MAXSTRING];
 BOOL  interactive;
 FOR_CONTEXT forloopcontext; /* The 'for' loop context */
 BOOL delayedsubst = FALSE; /* The current delayed substitution setting */
@@ -1099,41 +1099,24 @@ void WCMD_run_program (WCHAR *command, BOOL called)
              wine_dbgstr_w(stemofsearch));
   while (pathposn) {
     WCHAR  thisDir[MAX_PATH] = {'\0'};
-    int    length            = 0;
     WCHAR *pos               = NULL;
     BOOL  found             = FALSE;
-    BOOL inside_quotes      = FALSE;
 
     /* Work on the first directory on the search path */
-    pos = pathposn;
-    while ((inside_quotes || *pos != ';') && *pos != 0)
-    {
-        if (*pos == '"')
-            inside_quotes = !inside_quotes;
-        pos++;
-    }
-
-    if (*pos) { /* Reached semicolon */
+    pos = strchrW(pathposn, ';');
+    if (pos) {
       memcpy(thisDir, pathposn, (pos-pathposn) * sizeof(WCHAR));
       thisDir[(pos-pathposn)] = 0x00;
       pathposn = pos+1;
-    } else {    /* Reached string end */
+
+    } else {
       strcpyW(thisDir, pathposn);
       pathposn = NULL;
     }
 
-    /* Remove quotes */
-    length = strlenW(thisDir);
-    if (thisDir[length - 1] == '"')
-        thisDir[length - 1] = 0;
-
-    if (*thisDir != '"')
-        strcpyW(temp, thisDir);
-    else
-        strcpyW(temp, thisDir + 1);
-
     /* Since you can have eg. ..\.. on the path, need to expand
        to full information                                      */
+    strcpyW(temp, thisDir);
     GetFullPathNameW(temp, MAX_PATH, thisDir, NULL);
 
     /* 1. If extension supplied, see if that file exists */
@@ -2271,21 +2254,12 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
 
         } while (*extraData == 0x00);
         curPos = extraSpace;
-
-        /* Skip preceding whitespace */
-        while (*curPos == ' ' || *curPos == '\t') curPos++;
-
-        /* Replace env vars if in a batch context */
-        if (context) handleExpansion(curPos, FALSE, FALSE);
-
+        if (context) handleExpansion(extraSpace, FALSE, FALSE);
         /* Continue to echo commands IF echo is on and in batch program */
-        if (context && echo_mode && *curPos && *curPos != '@') {
+        if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
           WCMD_output_asis(extraSpace);
           WCMD_output_asis(newlineW);
         }
-
-        /* Skip repeated 'no echo' characters and whitespace */
-        while (*curPos == '@' || *curPos == ' ' || *curPos == '\t') curPos++;
       }
     }
 
